using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace Shader00
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Model[] testModels;
        int activeModel;
        Matrix world, view, projection;

        KeyboardState lastKbState;

        Vector3 cameraPos = new Vector3(0, 10, 5);

        Effect simplest;

        float theta = MathHelper.PiOver2, phi = 0;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            simplest = Content.Load<Effect>("Simplest");

            testModels = new Model[5];

            //load meshes
            testModels[0] = Content.Load<Model>("Cube");
            testModels[1] = Content.Load<Model>("SphereHighPoly");
            testModels[2] = Content.Load<Model>("SphereLowPoly");
            testModels[3] = Content.Load<Model>("Cylinder");
            testModels[4] = Content.Load<Model>("Cone");

            foreach (Model m in testModels)
                ChangeEffect(m, simplest);

            float aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.PiOver4,
                aspectRatio, .1f, 1000f);

            world = Matrix.Identity;

            view = Matrix.CreateLookAt(cameraPos, Vector3.Zero, Vector3.Up);

            simplest.Parameters["World"].SetValue(world);
            simplest.Parameters["View"].SetValue(view);
            simplest.Parameters["Projection"].SetValue(projection);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbState = Keyboard.GetState();
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            if (kbState.IsKeyDown(Keys.Space) && lastKbState.IsKeyUp(Keys.Space))
                activeModel = (activeModel + 1) % testModels.Length;

            if (kbState.IsKeyDown(Keys.Left))
                phi += (float)(gameTime.ElapsedGameTime.TotalSeconds);
            else if (kbState.IsKeyDown(Keys.Right))
                phi -= (float)(gameTime.ElapsedGameTime.TotalSeconds);

            if (kbState.IsKeyDown(Keys.Up))
                theta += (float)(gameTime.ElapsedGameTime.TotalSeconds);
            else if (kbState.IsKeyDown(Keys.Down))
                theta -= (float)(gameTime.ElapsedGameTime.TotalSeconds);

            lastKbState = kbState;

            cameraPos.X = (float)(5 * Math.Sin(theta) * Math.Sin(phi));
            cameraPos.Y = (float)(5 * Math.Cos(theta));
            cameraPos.Z = (float)(5 * Math.Sin(theta) * Math.Cos(phi));

            view = Matrix.CreateLookAt(cameraPos, Vector3.Zero, Vector3.Up);
            simplest.Parameters["View"].SetValue(view);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            DrawModel(testModels[activeModel]);

            base.Draw(gameTime);
        }

        public void DrawModel(Model model)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                graphics.GraphicsDevice.Indices = mesh.IndexBuffer;
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    DrawPart(mesh, part);
                }
            }
        }

        public void DrawPart(ModelMesh mesh, ModelMeshPart part)
        {
            part.Effect.Begin();

            foreach (EffectPass pass in part.Effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.VertexDeclaration = part.VertexDeclaration;
                graphics.GraphicsDevice.Vertices[0].SetSource(
                    mesh.VertexBuffer,
                    part.StreamOffset,
                    part.VertexStride
                );

                graphics.GraphicsDevice.DrawIndexedPrimitives(
                    PrimitiveType.TriangleList,
                    part.BaseVertex, 0,
                    part.NumVertices,
                    part.StartIndex,
                    part.PrimitiveCount
                );
                pass.End();
            }

            part.Effect.End();
        }

        public void ChangeEffect(Model model, Effect effect)
        {
            foreach (ModelMesh mesh in model.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = effect;
                }
            }
        }

    }
}
